import json
import logging
import os
import rsa
import base64
import requests

from ep_sdk_4pd import models as ep_sdk_4pd_models
from datetime import datetime

from ep_sdk_4pd.models import ModelOutputDirRequest, RunStrategyRequest, CallTrainDoneRequest, RsaKeyRequest

# test 地址
endpoint = 'http://172.27.88.56:6001'

# prod 地址
# endpoint = 'http://172.27.88.56:6601'

# 外网 地址
# endpoint = 'http://82.157.231.254:6001'

Authorization = 'Bearer eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJpc3MiOiJlbGVjdHJpY2l0eS1wbGF0Zm9ybSIsInN1YiI6IjEyMyIsImlhdCI6MTc0NjYwNjQ4NSwianRpIjoiMTIzXzE3NDY1Nzc2ODUxNDYiLCJ0eXBlIjoiYWNjZXNzIn0.Clrz_8j3aJlXTWPX-4DS0NxXN9idTcUIc0AtXOMIjd8'

def rsa_decrypt(privkey, ciphertext):
    decrypted = rsa.decrypt(base64.b64decode(ciphertext), privkey)
    return decrypted.decode('utf-8')

class EpSystem:

    @staticmethod
    def model_output_dir(is_online: bool = True):
        if is_online:
            # 从环境变量中获取策略id
            strategy_id = os.getenv('STRATEGY_ID')
            logging.info(f'strategy_id: {strategy_id}')

            if strategy_id is None:
                raise Exception('STRATEGY_ID is not set')
        else:
            # 线下环境，给固定的策略id
            strategy_id = 63
        request = ModelOutputDirRequest(strategy_id=strategy_id)
        full_url = f'{endpoint}{request.api}'
        headers = {
            'content-type': request.content_type,
            'Authorization': Authorization
        }

        payload = {
            'strategy_id': request.strategy_id
        }

        response = requests.request(
            method=request.method,
            url=full_url,
            headers=headers,
            data=json.dumps(payload),
        )

        base_resp = ep_sdk_4pd_models.BaseResponse(
            code=response.json().get('code', None),
            data=response.json().get('data', None),
            message=response.json().get('message', None),
        )
        response = ep_sdk_4pd_models.ModelOutputDirResponse(response=base_resp)

        if response.code == 200:
            return response.data
        else:
            return None

    @staticmethod
    def get_system_date(is_online: bool = True):
        """
        脚本每次预测的目标日，主要是拦截用户获取越界数据,用户不可修改
        """
        if is_online:
            # 线上环境,随着真实调用运行时间变化
            system_date = datetime.now().strftime('%Y-%m-%d')

            target_date = os.environ.get('TARGET_DATE')
            # target_date = 'JgAgrcfaQBI0h5YtLGGCjH92snKB3pUBOQs8yy0/mxMBdur0jNvAz6LzpU+Vq3QXsf36jy79WCrgVFo+0RwfUN338VJ9Qe+HwPfe0eq29De6eLD3nOl8xORj339dtSEdXq4dwc2mTaIxQc+erpv4acgQ7YVSDX6AuprwfW7c7qxthziqCqHxVm/qZwUkMsQ1fgzVLeICdscbCJbCD5GFYSpohaP5hRebutoa5FqyhYURA/4kxus98lJsENhnOZAACFdnCWDklpkKZyyOq0aKMd8Vv9SWX1LAZVTRjQ1GGuUPyOYjvnhUyNeRo3D85ODGBFeEju8rjDEB8LFSvX5d3Q=='
            if target_date is not None:
                rsa_data = EpSystem.get_rsa_key()
                if rsa_data:
                    rsa_private_key = rsa_data['rsa_private_key']
                    loaded_privkey = rsa.PrivateKey.load_pkcs1(rsa_private_key.encode('utf-8'))
                    decrypted_data = rsa_decrypt(loaded_privkey, target_date)
                    system_date = decrypted_data
        else:
            # 线下环境
            system_date = "2024-12-31"

        return system_date

    @staticmethod
    def get_run_strategy(is_online: bool = True):
        """
        获取此刻运行的策略模型基础信息
        :param is_online:
        :return:
        """
        if is_online:
            # 从环境变量中获取策略id
            strategy_id = os.getenv('STRATEGY_ID')
            logging.info(f'strategy_id: {strategy_id}')

            if strategy_id is None:
                raise Exception('STRATEGY_ID is not set')
        else:
            # 线下环境，给固定的策略id
            strategy_id = 1

        request = RunStrategyRequest(strategy_id=strategy_id)
        full_url = f'{endpoint}{request.api}'
        headers = {
            'content-type': request.content_type,
            'Authorization': Authorization
        }

        payload = {
            'strategy_id': request.strategy_id
        }

        response = requests.request(
            method=request.method,
            url=full_url,
            headers=headers,
            data=json.dumps(payload),
        )

        base_resp = ep_sdk_4pd_models.BaseResponse(
            code=response.json().get('code', None),
            data=response.json().get('data', None),
            message=response.json().get('message', None),
        )
        response = ep_sdk_4pd_models.RunStrategyResponse(response=base_resp)

        if response.code == 200:
            return response.data
        else:
            return None

    @staticmethod
    def call_train_done(
            strategy_id: int = None,
            script_strategy_id: int = None
    ):
        if (strategy_id is None
                or script_strategy_id is None
        ):
            strategy_id = os.getenv('STRATEGY_ID')
            script_strategy_id = os.getenv('SCRIPT_STRATEGY_ID')

        request = CallTrainDoneRequest(strategy_id=strategy_id,
                                       script_strategy_id=script_strategy_id)
        full_url = f'{endpoint}{request.api}'
        headers = {
            'content-type': request.content_type,
            'Authorization': Authorization
        }

        payload = {
            'strategy_id': request.strategy_id,
            'script_strategy_id': request.script_strategy_id
        }

        response = requests.request(
            method=request.method,
            url=full_url,
            headers=headers,
            data=json.dumps(payload),
        )

        base_resp = ep_sdk_4pd_models.BaseResponse(
            code=response.json().get('code', None),
            data=response.json().get('data', None),
            message=response.json().get('message', None),
        )
        response = ep_sdk_4pd_models.CallTrainDoneResponse(response=base_resp)

        if response.code == 200:
            return True
        else:
            return False

    @staticmethod
    def get_rsa_key():
        request = RsaKeyRequest()
        response = EpSystem.rsa_key(request=request)

        if response.code == 200:
            return response.data
        else:
            return None

    @staticmethod
    def rsa_key(
            request: ep_sdk_4pd_models.RsaKeyRequest = None,
    ) -> ep_sdk_4pd_models.RsaKeyResponse:

        full_url = f'{endpoint}{request.api}'
        headers = {
            'content-type': request.content_type,
            'Authorization': Authorization
        }

        response = requests.request(
            method=request.method,
            url=full_url,
            headers=headers,
        )

        base_resp = ep_sdk_4pd_models.BaseResponse(
            code=response.json().get('code', None),
            data=response.json().get('data', None),
            message=response.json().get('message', None),
        )
        return ep_sdk_4pd_models.RsaKeyResponse(response=base_resp)


from ep_sdk_4pd.ep_system import EpSystem


def test_get_system_date():
    print('-------------test_get_system_date-------------')

    target_date = EpSystem.get_system_date()
    print(target_date)
    print('-------------------------------------')


if __name__ == '__main__':
    test_get_system_date()



from ep_sdk_4pd.ep_system import EpSystem


def test_call_train_done():
    print('-------------test_call_train_done-------------')

    data = EpSystem.call_train_done(strategy_id=76,script_strategy_id=27)
    print(data)
    print('-------------------------------------')


if __name__ == '__main__':
    test_call_train_done()



from ep_sdk_4pd.ep_data import EpData


def test_history_data():
    print('-------------test_history_data-------------')

    data = EpData.get_history_data(days=2)
    print(data)
    print('-------------------------------------')


if __name__ == '__main__':
    test_history_data()



import json
from datetime import timedelta, datetime

import requests

from ep_sdk_4pd import models as ep_sdk_4pd_models
from ep_sdk_4pd.ep_system import EpSystem
from ep_sdk_4pd.models import HistoryDataRequest, PredictDataRequest

# test 地址
endpoint = 'http://172.27.88.56:6001'

# prod 地址
# endpoint = 'http://172.27.88.56:6601'

# 外网 地址
# endpoint = 'http://82.157.231.254:6001'

Authorization = 'Bearer eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJpc3MiOiJlbGVjdHJpY2l0eS1wbGF0Zm9ybSIsInN1YiI6IjEyMyIsImlhdCI6MTc0NjYwNjQ4NSwianRpIjoiMTIzXzE3NDY1Nzc2ODUxNDYiLCJ0eXBlIjoiYWNjZXNzIn0.Clrz_8j3aJlXTWPX-4DS0NxXN9idTcUIc0AtXOMIjd8'


class EpData:

    @staticmethod
    def get_history_data(
            scope="weather,plant,market",
            days=0
    ):
        # 最晚时间为系统时间 D-2
        date_str = EpSystem.get_system_date(is_online=True)
        # 许涛最晚时间为 D-2
        # calculated_date = datetime.strptime(date_str, "%Y-%m-%d") - timedelta(days=2)
        # 连学最晚时间为 D-1
        calculated_date = datetime.strptime(date_str, "%Y-%m-%d") - timedelta(days=1)
        system_date = calculated_date.strftime("%Y-%m-%d")  # 转换回字符串

        request = HistoryDataRequest(
            scope=scope,
            system_date=system_date,
            days=days,
        )
        response = EpData.history_data(request=request)

        if response.code == 200:
            return response.data
        else:
            return None

    @staticmethod
    def history_data(
            request: ep_sdk_4pd_models.HistoryDataRequest = None,
    ) -> ep_sdk_4pd_models.HistoryDataResponse:

        full_url = f'{endpoint}{request.api}'
        headers = {
            'content-type': request.content_type,
            'Authorization': Authorization
        }

        payload = {
            'scope': request.scope,
            'system_date': request.system_date,
            'days': request.days
        }

        response = requests.request(
            method=request.method,
            url=full_url,
            headers=headers,
            data=json.dumps(payload),
        )

        base_resp = ep_sdk_4pd_models.BaseResponse(
            code=response.json().get('code', None),
            data=response.json().get('data', None),
            message=response.json().get('message', None),
        )
        return ep_sdk_4pd_models.HistoryDataResponse(response=base_resp)

    @staticmethod
    def get_predict_data(
            scope="weather,plant,market",
            is_test=False,
            test_time=None,
    ):
        date_str = EpSystem.get_system_date(is_online=True)
        calculated_date = datetime.strptime(date_str, "%Y-%m-%d") + timedelta(days=1)  # 增加 +1 天
        system_date = calculated_date.strftime("%Y-%m-%d")

        # 测试
        # if is_test:
        #     system_date = test_time

        request = PredictDataRequest(
            scope=scope,
            system_date=system_date,
        )
        response = EpData.predict_data(request=request)

        if response.code == 200:
            return response.data
        else:
            return None

    @staticmethod
    def predict_data(
            request: ep_sdk_4pd_models.PredictDataRequest = None,
    ) -> ep_sdk_4pd_models.PredictDataResponse:

        full_url = f'{endpoint}{request.api}'
        headers = {
            'content-type': request.content_type,
            'Authorization': Authorization
        }

        payload = {
            'scope': request.scope,
            'system_date': request.system_date
        }

        response = requests.request(
            method=request.method,
            url=full_url,
            headers=headers,
            data=json.dumps(payload),
        )

        base_resp = ep_sdk_4pd_models.BaseResponse(
            code=response.json().get('code', None),
            data=response.json().get('data', None),
            message=response.json().get('message', None),
        )
        return ep_sdk_4pd_models.PredictDataResponse(response=base_resp)



from ep_sdk_4pd.ep_system import EpSystem


def test_get_run_strategy():
    print('-------------test_get_run_strategy-------------')

    data = EpSystem.get_run_strategy(is_online=True)
    print(data)
    print('-------------------------------------')


if __name__ == '__main__':
    test_get_run_strategy()



from ep_sdk_4pd.ep_system import EpSystem


def test_model_output_dir():
    print('-------------test_model_output_dir-------------')

    data = EpSystem.model_output_dir(is_online=True)
    print(data)
    print('-------------------------------------')


if __name__ == '__main__':
    test_model_output_dir()



import os

from setuptools import setup, find_packages

PACKAGE = 'ep_sdk_4pd'
NAME = 'ep_sdk_4pd'
DESCRIPTION = '4paradigm Electricity Platform Service SDK Library for Python'
AUTHOR = '4paradigm Electricity Platform SDK'
AUTHOR_EMAIL = ''
URL = 'https://gitlab.4pd.io/electricityproject/electricity-platform-sdk'
VERSION = '1.1.0.dev0'
REQUIRES = ['requests']

LONG_DESCRIPTION = ''
if os.path.exists('./README.md'):
    with open('README.md', encoding='utf-8') as fp:
        LONG_DESCRIPTION = fp.read()

setup(
    name=NAME,
    version=VERSION,
    description=DESCRIPTION,
    long_description=LONG_DESCRIPTION,
    long_description_content_type='text/markdown',
    author=AUTHOR,
    author_email=AUTHOR_EMAIL,
    # license='Apache License 2.0',
    url=URL,
    keywords=['4pd_electricity_platform'],
    packages=find_packages(exclude=['tests*']),
    include_package_data=True,
    platforms='any',
    install_requires=REQUIRES,
    python_requires='>=3.6',
    classifiers=[
        # 'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        # 'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Topic :: Software Development',
    ],
)



from ep_sdk_4pd.ep_data import EpData


def test_predict_data():
    print('-------------test_predict_data-------------')

    data = EpData.get_predict_data(scope="plant")
    print(data)
    print('-------------------------------------')


if __name__ == '__main__':
    test_predict_data()



class BaseRequest:
    """
    Model for BaseRequest
    """

    def __init__(self):
        self.api = None
        self.method = None
        self.content_type = None
        self.payload = None


class BaseResponse:
    """
    Model for BaseResponse
    """

    def __init__(
            self, code: int = None, data: dict = None, message: str = None, **kwargs
    ):
        self.code = code
        self.data = data
        self.message = message


class HistoryDataRequest(BaseRequest):
    """
    Model for HistoryDataRequest

    获取具体系统时间往前x天的数据（最晚时间为系统时间 D-2）
    特别注意：get_history_data 或 get_predict_data 会get_system_date进行检测，不可获取system_date以后得数据，避免数据穿越现象
    """

    def __init__(self, scope: str = None, system_date: str = None, days: int = None):
        """
        Args:
            scope: "weather","plant","market"
            system_date: 系统时间
            days: 表示systemDate之前days的数据
        """

        self.scope = scope
        self.system_date = system_date
        self.days = days

        super().__init__()
        self.api = f'/ep/api/sdk/get_history_data'
        self.method = 'POST'
        self.content_type = 'application/json'


class HistoryDataResponse(BaseResponse):
    """
    Model for HistoryDataResponse
    """

    def __init__(self, response: BaseResponse = None, **kwargs):
        super().__init__(
            code=response.code if response else None,
            data=response.data if response else None,
            message=response.message if response else None,
            **kwargs,
        )

class PredictDataRequest(BaseRequest):
    """
    Model for PredictDataRequest

    获取当前系统时间的预测数据（按照抓取时间，最晚时间为当前系统时间的d-1
    """

    def __init__(self, scope: str = None, system_date: str = None):
        """
        Args:
            scope: "weather","market"
            system_date: 系统时间
        """

        self.scope = scope
        self.system_date = system_date

        super().__init__()
        self.api = f'/ep/api/sdk/get_predict_data'
        self.method = 'POST'
        self.content_type = 'application/json'


class PredictDataResponse(BaseResponse):
    """
    Model for PredictDataResponse
    """

    def __init__(self, response: BaseResponse = None, **kwargs):
        super().__init__(
            code=response.code if response else None,
            data=response.data if response else None,
            message=response.message if response else None,
            **kwargs,
        )

class ModelOutputDirRequest(BaseRequest):
    """
    Model for ModelOutputDirRequest

    输出各策略的存放位置
    输入：
    线上环境：
    ${user.home}/strategyId/

    本地测试：
    ${user.home}/test/
    """

    def __init__(self, strategy_id: int = None):
        """
        Args:
            strategy_id: 是否线上环境
        """
        self.strategy_id = strategy_id

        super().__init__()
        self.api = f'/ep/api/sdk/model_output_dir'
        self.method = 'POST'
        self.content_type = 'application/json'


class ModelOutputDirResponse(BaseResponse):
    """
    Model for ModelOutputDirResponse
    """

    def __init__(self, response: BaseResponse = None, **kwargs):
        super().__init__(
            code=response.code if response else None,
            data=response.data if response else None,
            message=response.message if response else None,
            **kwargs,
        )

class RunStrategyRequest(BaseRequest):
    """
    Model for RunStrategyRequest

    获取此刻运行的策略模型基础信息
    """

    def __init__(self, strategy_id: int = None):
        """
        Args:
            strategy_id: 策略 id
        """
        self.strategy_id = strategy_id

        super().__init__()
        self.api = f'/ep/api/sdk/get_run_strategy'
        self.method = 'POST'
        self.content_type = 'application/json'


class RunStrategyResponse(BaseResponse):
    """
    Model for RunStrategyResponse
    """

    def __init__(self, response: BaseResponse = None, **kwargs):
        super().__init__(
            code=response.code if response else None,
            data=response.data if response else None,
            message=response.message if response else None,
            **kwargs,
        )

class CallTrainDoneRequest(BaseRequest):
    """
    Model for CallTrainDoneRequest

    回调设置模型策略训练完成，考虑是框架层调用 还是 用户自由设定
    输入：无，默认为EpSystem.get_run_strategy()["id"]
    """

    def __init__(
            self,
            strategy_id: int = None,
            script_strategy_id: int = None
    ):
        """
        Args:
            strategy_id
            script_strategy_id
        """
        self.strategy_id = strategy_id
        self.script_strategy_id = script_strategy_id

        super().__init__()
        self.api = f'/ep/api/sdk/call_train_done'
        self.method = 'POST'
        self.content_type = 'application/json'


class CallTrainDoneResponse(BaseResponse):
    """
    Model for CallTrainDoneResponse
    """

    def __init__(self, response: BaseResponse = None, **kwargs):
        super().__init__(
            code=response.code if response else None,
            data=response.data if response else None,
            message=response.message if response else None,
            **kwargs,
        )

class RsaKeyRequest(BaseRequest):
    """
    Model for RsaKeyRequest
    """

    def __init__(
            self
    ):

        super().__init__()
        self.api = f'/ep/api/sdk/get_rsa_key'
        self.method = 'GET'
        self.content_type = 'application/json'


class RsaKeyResponse(BaseResponse):
    """
    Model for RsaKeyResponse
    """

    def __init__(self, response: BaseResponse = None, **kwargs):
        super().__init__(
            code=response.code if response else None,
            data=response.data if response else None,
            message=response.message if response else None,
            **kwargs,
        )